/* 
**************************************************** 
* @brief MU Vision Sensor

 * @copyright    [DFRobot](https://www.dfrobot.com), 2016
 * @copyright    GNU Lesser General Public License

* @author [WWZ](Wenzheng.wang@dfrobot.com)
* @version  V1.0
* @date  2019-03-11

* GNU Lesser General Public License.
* All above must be included in any redistribution
* ***************************************************
*/

#include <Wire.h>
#include "Servo.h"
#include "motor.h"

const int motorRENPin = 10;
const int motorRIN2Pin = 9;
const int motorRIN1Pin = 8;
const int motorLENPin = 5;
const int motorLIN2Pin = 7;
const int motorLIN1Pin = 6;

Motor rightMotor(motorRIN1Pin, motorRIN2Pin, motorRENPin);
Motor leftMotor(motorLIN1Pin, motorLIN2Pin, motorLENPin);

#define MU_ADDRESS    0x60
#define PROTOCOL_VER  0x03
#define VISION_ID     0x03      // 0x03 for ball, 0x05 for body 

// register define
#define REG_PROTOCOL_VER  0x01
#define REG_LED1_CONF     0x06
#define REG_LED2_CONF     0x07
#define REG_LED_LEVEL     0x08
#define REG_CAMERA_CONF1  0x10
#define REG_FRAME_CNT     0x1F
#define REG_VISION_ID     0x20
#define REG_VISION_CONF1  0x21
#define REG_PARAM_VALUE5  0x29
#define RESULT_NUM        0x34
#define RESULT_DATA1      0x40
#define RESULT_DATA2      0x41
#define RESULT_DATA3      0x42
#define RESULT_DATA4      0x43
#define RESULT_DATA5      0x44
// color
#define MU_COLOR_BLACK                0x01U
#define MU_COLOR_WHITE                0x02U
#define MU_COLOR_RED                  0x03U
#define MU_COLOR_YELLOW               0x04U
#define MU_COLOR_GREEN                0x05U
#define MU_COLOR_CYAN                 0x06U
#define MU_COLOR_BLUE                 0x07U
#define MU_COLOR_PURPLE               0x08U

int i2c_read8(uint8_t reg) {
  Wire.beginTransmission(MU_ADDRESS);
  Wire.write(reg);
  Wire.endTransmission();

  Wire.requestFrom(MU_ADDRESS, 1);
  return Wire.read();
}
void i2c_write8(const uint8_t reg, const uint8_t value) {
  Wire.beginTransmission(MU_ADDRESS);
  Wire.write(reg);
  Wire.write(value);
  Wire.endTransmission();
}

uint8_t reg[][2] = {
    { REG_VISION_ID,      VISION_ID }, // set vision type = vision_detect
    { REG_VISION_CONF1,   0x21 }, // vision begin
};
uint8_t frame_count_last = 0;

Servo servo1; //Base
Servo servo2; //Upper

int angle1 = 90;
int angle2 = 130;

int xPos;
int yPos;

int width;
int height;

int rotSpeed = 4;
int turnSpeed = 255;

void setup() {
  // put your setup code here, to run once:
  Serial.begin(9600);
  Wire.begin();

  servo1.attach(3);
  servo2.attach(11);

  servo1.write(angle1);
  servo2.write(angle2);
  delay(500);

  if (i2c_read8(REG_PROTOCOL_VER) == PROTOCOL_VER) {
    Serial.println("device initialized.");
  } else {
    Serial.println("fail to initialize device! Please check protocol version.");
  }
  for (uint32_t i = 0; i < sizeof(reg)/2; ++i) {
    i2c_write8(reg[i][0], reg[i][1]);
  }
}

void loop() {
  // put your main code here, to run repeatedly:
  long time_start = millis();
  int frame_count = 0;
  // waiting for update
  do {
    frame_count = i2c_read8(REG_FRAME_CNT);
  } while(frame_count == frame_count_last);
  frame_count_last = frame_count;

  i2c_write8(REG_VISION_ID, VISION_ID);
  // read result
  if (i2c_read8(RESULT_NUM) > 0) {
    //Serial.println("body detected:");

    xPos = i2c_read8(RESULT_DATA1);
    yPos = i2c_read8(RESULT_DATA2);

    width = i2c_read8(RESULT_DATA3);
    height = i2c_read8(RESULT_DATA4);

    if(angle1 > 180)
    {
      angle1 = 180;  
    }

    if(angle1 < 0)
    {
      angle1 = 0;  
    }

    if(angle2 > 180)
    {
      angle2 = 180;  
    }

    if(angle2 < 0)
    {
      angle2 = 0;  
    }
    
    if(xPos < 45 && xPos < 50)
    {
      //rotSpeed = xPos / 15;
      angle1 = angle1 + rotSpeed;
      servo1.write(angle1);
      //Serial.println("1");
    }

    if(xPos > 50 && xPos > 55)
    {
      angle1 = angle1 - rotSpeed;
      servo1.write(angle1);
    }

    if(yPos < 40)
    {
      angle2 = angle2 - rotSpeed;
      servo2.write(angle2);
    }

    if(yPos > 50)
    {
      angle2 = angle2 + rotSpeed;
      servo2.write(angle2);
    }

    if(angle1 >= 70 && angle1 <= 110)
    {
       rightMotor.backward(255);
       leftMotor.backward(255);  
    }

    if(angle1 > 110)
    {
      leftMotor.forward(turnSpeed);
      rightMotor.backward(turnSpeed);   
    }

    if(angle1 < 70)
    {
      leftMotor.backward(turnSpeed);
      rightMotor.forward(turnSpeed);   
    }
    
  }
  
  else 
  {
    rightMotor.brake();
    leftMotor.brake(); 
  }
}
